/*
 This program is free software: you can use, modify and/or
 redistribute it under the terms of the simplified BSD
 License. You should have received a copy of this license along
 this program. If not, see
 <http://www.opensource.org/licenses/bsd-license.html>.
 */


#include <inout.h>

/**
 * @file    inout.cpp
 * @brief   This file implements the functions necessary 
 *          to read/write TIFF images
 */


/**
 * @brief Writing image to the disc
 *
 * @param archima
 *      File location to write the image into the disc
 * @param imR
 *      RGB-Image to be written into the disc
 */
void write(char * archima, Image * imR)
{
    // This function writes the low-range image imR in the file 'archima',
    double *data_out;
    size_t nx = imR->rows(), ny = imR->cols();
    
    // limit to 0-255
    data_out = (double*)malloc(nx*ny*4*sizeof(double));
    
    int k=0;
    double maxval = 0,minval=1000;
    for (int c = 0; c<3; c++)
        for (int i =0;i<nx*ny;i++){
            data_out[k] = min(max(imR[c].data[i],0),1)*255;
            k++;
        }
    //alpha channel
    for (int i =0;i<nx*ny;i++){
        data_out[k] =255;
        k++;
    }
    
    // TIFF RGBA double->8bit output
    write_tiff_rgba_f32(archima, data_out, ny, nx);
    free(data_out);
}
/**
 * @brief Writing image to the disc
 *
 * @param archima
 *      File location to write the image into the disc
 * @param imR
 *      BW-Image to be written into the disc
 */
void writeBW(char * archima, Image * imR)
{
    // This function writes the low-range image imR
    // in the file 'archima', assuming ths input image imR
    // is black and white
    double *data_out;
    size_t nx = imR->rows(), ny = imR->cols();
    
    // limit to 0-255
    data_out = (double*)malloc(nx*ny*4*sizeof(double));
    
    int k=0;
    double maxval = 0,minval=1000;
    for (int c = 0; c<3; c++)
        for (int i =0;i<nx*ny;i++){
            data_out[k] = min(max(imR->data[i],0),1)*255;
            k++;
        }
    //alpha channel
    for (int i =0;i<nx*ny;i++){
        data_out[k] =255;
        k++;
    }
    
    // TIFF RGBA double->8bit output
    write_tiff_rgba_f32(archima, data_out, ny, nx);
    free(data_out);
}


/**
 * @brief Reading image from the disc
 *
 * @param out
 *      Image object where the read image will be staraged
 * @param archima
 *      File location to read the image from the disc
 *
 */
void read(Image* out, char * archima)
{
    size_t nx=0,ny=0;
    unsigned char *data_in=NULL;
    
    data_in = read_tiff_rgba_u8(archima, &nx, &ny);
  
    for (int color =0; color < 3; color++)
    {
        out[color].initialize((int)ny,(int)nx);
        
        for (int i = 0; i < nx*ny; i++)
            out[color].data[i]= (double)data_in
                                [(nx*ny)*color+i]/255;
            //values are in [0,1]
    }
  
    free(data_in);
}

